
#include <stdint.h>

#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

#include "LPC17xx.h"
#include "led.h"


#define LED_QUEUE_LENGTH                        (4)


xQueueHandle led_queue;

/** Simple task to control the LED, mainly used as keep-alive indicator. */
void led_task (void *pvParameters __attribute__((unused)))
{
    portTickType last_time;
    portTickType on_time = 100;
    portTickType off_time = 100;
    uint32_t pattern;
    uint8_t pattern_length;
    struct structLedQueueElem message;
    uint32_t i;
    
    
    /* We need to initialise xLastFlashTime prior to the first call to vTaskDelayUntil(). */
    last_time = xTaskGetTickCount();
    
    /* Start in normal mode */
    message.opcode = LED_OPCODE_BLINK_NORMAL;
    xQueueSendToBack(led_queue, &message, 0);

    while (1)
    {
        /* Is there a new message? (non-blocking) */
        if (xQueueReceive( led_queue, &message, 0))
        {
            switch (message.opcode) {
            case LED_OPCODE_BLINK_NORMAL:
                on_time = (portTickType) 400 / portTICK_RATE_MS;
                off_time = (portTickType) 400 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;

            case LED_OPCODE_BLINK_WARNING:
                on_time = (portTickType) 100 / portTICK_RATE_MS;
                off_time = (portTickType) 100 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;
            }
        }

        /* Run once through LED pattern. */
        for (i = 0; i < pattern_length; i++) {
            if (pattern & (1UL << i)) {
                /* LED on */
                LPC_GPIO1->FIOCLR = (1 << 18);
                vTaskDelayUntil (&last_time, on_time);
            }
            else {
                /* LED off */
                LPC_GPIO1->FIOSET = (1 << 18);
                vTaskDelayUntil (&last_time, off_time);
            }
        }
    }
}


/** LED module init function.
 *
 *  Must be called before the scheduler starts. Initializes the message queue.
 */
void led_init_module (void)
{
    /* Task communication */
    led_queue = xQueueCreate ((unsigned portBASE_TYPE) LED_QUEUE_LENGTH,
                              (unsigned portBASE_TYPE)  sizeof (struct structLedQueueElem));
}
