#ifndef __USBCLASS_H__
#define __USBCLASS_H__


#include <stdint.h>
#include <stdbool.h>

#include "FreeRTOS.h"
#include "queue.h"

#include "usbstruct.h"


/* Descriptor type */
#define DESC_INTERFACE_ASSOCIATION    11

/* Audio class requests */
#define SET_CUR                       (0x01)
#define GET_CUR                       (0x81)
#define SET_MIN                       (0x02)
#define GET_MIN                       (0x82)
#define SET_MAX                       (0x03)
#define GET_MAX                       (0x83)
#define SET_RES                       (0x04)
#define GET_RES                       (0x84)
#define SET_MEM                       (0x05)
#define GET_MEM                       (0x85)
#define GET_STAT                      (0xFF)

/* Audio Class Control Selectors */
#define USBAC_CS_FU_MUTE_CONTROL                  (0x01)
#define USBAC_CS_FU_VOLUME_CONTROL                (0x02)
#define USBAC_CS_FU_BASS_CONTROL                  (0x03)
#define USBAC_CS_FU_MID_CONTROL                   (0x04)
#define USBAC_CS_FU_TREBLE_CONTROL                (0x05)
#define USBAC_CS_FU_GRAPHIC_EQUALIZER_CONTROL     (0x06)
#define USBAC_CS_FU_AUTOMATIC_GAIN_CONTROL        (0x07)
#define USBAC_CS_FU_DELAY_CONTROL                 (0x08)
#define USBAC_CS_FU_BASS_BOOST_CONTROL            (0x09)
#define USBAC_CS_FU_LOUDNESS_CONTROL              (0x0A)

#define USBAC_CS_EP_SAMPLING_FREQ_CONTROL         (0x01)
#define USBAC_CS_EP_PITCH_CONTROL                 (0x02)



/*************************************************************************
	USB application interface
**************************************************************************/

/* USB task */
void usb_task (void *pvParameters);


#define USBQUEUE_LENGTH                 (16)
typedef struct {
    uint8_t         opcode;
    TSetupPacket    setup;
    void            **ppData;
    uint16_t        frame;
    int16_t         volume;
    uint8_t         mode;
} usb_Message;
extern xQueueHandle usb_queue;

#define USBMSG_I2S_START                (2)
#define USBMSG_I2S_STOP                 (3)
#define USBMSG_I2S_CHANGE_SRATE         (4)
#define USBMSG_SETUP_REQUEST            (5)
#define USBMSG_FRAME                    (6)
#define USBMSG_I2S_HOLD                 (7)
#define USBMSG_SET_VOLUME               (8)
#define USBMSG_SET_PREAMP               (15)


int16_t usbclass_getVolume (void);
void usbclass_setupRequestFromISR (TSetupPacket *pSetup, void **ppData, portBASE_TYPE *taskWoken);
void usbclass_frameFromISR (uint16_t frame, portBASE_TYPE *taskWoken);

#endif

