/* USB secondary boot code for LPC1700 Cortex-M3 microcontrollers
 *
 * Copyright 2010 DF9DQ
 *
 * This file is part of the LPC1700 USB secondary boot code (Boot1700-USB).
 *
 * Boot1700-USB is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Boot1700-USB is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Boot1700-USB.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <inttypes.h>

/* Descriptor Types */
#define USBD_DEVICE                                     (1)
#define USBD_CONFIGURATION                              (2)
#define USBD_STRING                                     (3)
#define USBD_INTERFACE                                  (4)
#define USBD_ENDPOINT                                   (5)
#define USBD_INTERFACE_ASSOCIATION                      (11)
#define USBD_CS_DEVICE                                  (0x21)
#define USBD_CS_CONFIGURATION                           (0x22)
#define USBD_CS_STRING                                  (0x23)
#define USBD_CS_INTERFACE                               (0x24)
#define USBD_CS_ENDPOINT                                (0x25)


/* Length of descriptors (standard) */
#define USBD_SIZE_DEVICE                                (18)
#define USBD_SIZE_CONFIGURATION                         (9)
#define USBD_SIZE_INTERFACE                             (9)
#define USBD_SIZE_ENDPOINT                              (9)
#define USBD_SIZE_INTERFACE_ASSOCIATION                 (8)

/* Subtype and length of descriptors (Audio class) */
#define USBD_AUDIO_ST_HEADER                            (1)
#define USBD_SIZE_AUDIO_ST_HEADER(n)                    (8+(n))
#define USBD_AUDIO_ST_INPUT_TERMINAL                    (2)
#define USBD_SIZE_AUDIO_ST_INPUT_TERMINAL               (12)
#define USBD_AUDIO_ST_OUTPUT_TERMINAL                   (3)
#define USBD_SIZE_AUDIO_ST_OUTPUT_TERMINAL              (9)
#define USBD_AUDIO_ST_SELECTOR_UNIT                     (5)
#define USBD_SIZE_AUDIO_ST_SELECTOR_UNIT(p)             (6+(p))
#define USBD_AUDIO_ST_FEATURE_UNIT                      (6)
#define USBD_SIZE_AUDIO_ST_FEATURE_UNIT(ch,n)           (7+((ch)+1)*(n))


/* USB Class Codes and applicable sub classes and protocols */
#define USBC_BASE                                       (0x00)
#define USBC_AUDIO                                      (0x01)
#define   USBCS_AUDIO_AUDIOCONTROL                        (0x01)
#define   USBCS_AUDIO_AUDIOSTREAMING                      (0x02)
#define   USBCS_AUDIO_MIDISTREAMING                       (0x03)
#define USBC_CDC_CONTROL                                (0x02)
#define USBC_HID                                        (0x03)
#define USBC_PHYSICAL                                   (0x05)
#define USBC_IMAGE                                      (0x06)
#define USBC_PRINTER                                    (0x07)
#define USBC_MASS_STORAGE                               (0x08)
#define   USBCS_MASS_STORAGE_RBC                          (0x01)
#define   USBCS_MASS_STORAGE_MMC2                         (0x02)
#define   USBCS_MASS_STORAGE_QIC157                       (0x03)
#define   USBCS_MASS_STORAGE_UFI                          (0x04)
#define   USBCS_MASS_STORAGE_SFF8070I                     (0x05)
#define   USBCS_MASS_STORAGE_SCSI                         (0x06)
#define     USBCP_MASS_STORAGE_CBI_WITH                     (0x00)
#define     USBCP_MASS_STORAGE_CBI_WITHOUT                  (0x01)
#define     USBCP_MASS_STORAGE_BULK_ONLY                    (0x50)
#define USBC_HUB                                        (0x09)
#define USBC_CDC_DATA                                   (0x0A)
#define USBC_SMART_CARD                                 (0x0B)
#define USBC_CONTENT_SECURITY                           (0x0D)
#define USBC_VIDEO                                      (0x0E)
#define USBC_PERSONAL_HEALTHCARE                        (0x0F)
#define USBC_DIAGNOSTIC_DEVICE                          (0xDC)
#define USBC_WIRELESS_CONTROLLER                        (0xE0)
#define USBC_MISCELLANEOUS                              (0xEF)
#define USBC_APPLICATION_SPECIFIC                       (0xFE)
#define USBC_VENDOR_SPECIFIC                            (0xFF)


/* For convenience */
#define LE(w) ((w) % 256), ((w) / 256)

/* If you want the serial number string (device descriptor) to be replaced
 * by the 128-bit device serial number (as ASCII string), set this to the
 * same value as iSerialNumber in the device descriptor. If not, set to 0.
 */
#if 1
#define USB_SERIAL_STRING                               (3)
#else
#define USB_SERIAL_STRING                               (0)
#endif

/* Declare definitions from usbdesc.c */
extern const uint8_t usb_descriptors[];
