

#include "FreeRTOS/include/FreeRTOS.h"
#include "task.h"



#include "LPC17xx.h"
#include "init.h"
#include "led.h"

#include "lpcusb/usbclass.h"
#include "i2c/si570.h"
#include "params.h"



/* Task priorities. */
#define mainLED_TASK_PRIORITY         ( tskIDLE_PRIORITY | portPRIVILEGE_BIT )
#define mainUSB_TASK_PRIORITY         ( (tskIDLE_PRIORITY + 2) | portPRIVILEGE_BIT )
#define mainSi570_TASK_PRIORITY       ( (tskIDLE_PRIORITY + 1) | portPRIVILEGE_BIT )




void vApplicationStackOverflowHook (xTaskHandle *pxTask, signed char *pcTaskName);
void vApplicationIdleHook (void);



/* Definition of USB task with restricted resource access (MPU) */
#define USB_TASK_STACKSIZE      (4096)



/*-----------------------------------------------------------*/



int main (void)
{
    paramsRead();

    /* Setup PLL */
    cpuInit();

    /* Things we must do before we can start the scheduler. */
    led_init_module();
    si570_initModule();

    /* LED blinker */
    xTaskCreate (led_task, (signed char *) "LED", configMINIMAL_STACK_SIZE, (void *) NULL, mainLED_TASK_PRIORITY, NULL);

    /* USB task */
    xTaskCreate (usb_task, (signed char *) "USB", USB_TASK_STACKSIZE/4, (void *) NULL, mainUSB_TASK_PRIORITY, NULL);

    /* I2C task (Si570) */
    // TODO: Warum ist Stackverbrauch zu hoch? */
    xTaskCreate( si570_task, (signed char *) "Si570", 1024/4, (void *) NULL, mainSi570_TASK_PRIORITY, NULL );

    /* Start the scheduler. */
    vTaskStartScheduler();

    for (;;)
    {
    }
}


/*-----------------------------------------------------------*/


void vApplicationStackOverflowHook (xTaskHandle *pxTask __attribute__((unused)),
                                    signed char *pcTaskName __attribute__((unused)))
{
    volatile uint32_t i;

    /* Stack overflow in task 'pxTask' detected.
     * We cannot recover from this failure! We might even have been called from within
     * an interrupt context.
     *
     * Just let the LED blink very fast as an indicator.
     */

    while (1) {
        for (i = 0; i < 500000UL; i++) {
        }

        if (LPC_GPIO1->FIOPIN & (1 << 18)) {
            LPC_GPIO1->FIOCLR = (1 << 18);
        }
        else {
            LPC_GPIO1->FIOSET = (1 << 18);
        }
    }
}


void vApplicationIdleHook (void)
{
    /* Stop the CPU until the next interrupt */
    SCB->SCR |= SCB_SCR_SEVONPEND_Msk;
    __WFE();
}

